type Query = Record<string, any>;
type Body = URLSearchParams | FormData | Query;
type Result<T, Safe extends boolean> = Promise<Safe extends true ? [T, Error] : T>;
export type HttpMethod = "GET" | "POST" | "PUT" | "DELETE";
interface Client<Safe extends boolean> {
    call<T>(method: HttpMethod, url: string, qs?: Query, body?: Body): Result<T, Safe>;
    get<T>(url: string, qs?: Query, body?: Body): Result<T, Safe>;
    post<T>(url: string, qs?: Query, body?: Body): Result<T, Safe>;
    delete<T>(url: string, qs?: Query, body?: Body): Result<T, Safe>;
    postBody<T>(url: string, body: Body): Result<T, Safe>;
    put<T>(url: string, body: Body): Result<T, Safe>;
}
export interface Visitor {
    ok: <T>(response: Response) => Promise<T>;
    fail: <T>(request: Request, response: Response) => Promise<T>;
}
type WithCredentials = {
    init: RequestInit;
    url: string;
};
export interface CredentialsManager {
    refresh: () => Promise<void>;
    isValid: (req: Request, res: Response) => Promise<boolean>;
    attach: (init: RequestInit, url?: string) => WithCredentials | Promise<WithCredentials>;
}
type ClientFactory = {
    visitor: Visitor;
    credentials: CredentialsManager;
};
export declare function addQueryParamsToUrl(url: string, queryParams?: Query): string;
export declare function createApiClient<Safe extends boolean>(factory: ClientFactory, safe: Safe): Client<Safe>;
export {};
