export function addQueryParamsToUrl(url, queryParams = {}) {
    const parsedUrl = new URL(url, globalThis.location?.origin);
    url = url.replace(parsedUrl.search, "");
    let search = parsedUrl.searchParams;
    for (const [key, values] of Object.entries(queryParams)) {
        if (values !== undefined) {
            [].concat(values).forEach((value) => search.append(key, value));
        }
    }
    let qs = search.toString();
    return qs.length === 0 ? url : `${url}?${qs}`;
}
function createRequestInit(method, body) {
    let init = { method };
    if (body?.constructor?.name !== "Object") {
        init.body = body;
        return init;
    }
    init.headers = { "content-type": "application/json" };
    if (body) {
        init.body = JSON.stringify(body);
    }
    return init;
}
async function unwrap(cb, safe) {
    try {
        let ok = await cb();
        // @ts-ignore
        return safe ? [ok, null] : ok;
    }
    catch (cause) {
        let error = new Error(cause.message, { cause });
        if (safe) {
            // @ts-ignore
            return [null, error];
        }
        throw error;
    }
}
async function execute(credentials) {
    let request = new Request(credentials.url, credentials.init);
    let response = await fetch(request);
    return [request, response];
}
export function createApiClient(factory, safe) {
    const { visitor, credentials } = factory;
    async function call(method, url, qs = {}, body) {
        let init = createRequestInit(method, body);
        let withQs = addQueryParamsToUrl(url, qs);
        let [request, response] = await execute(await credentials.attach({ ...init }, withQs));
        if (response.ok) {
            return unwrap(() => visitor.ok(response), safe);
        }
        let isValid = await credentials.isValid(request, response);
        if (!isValid) {
            let refresh = await unwrap(() => credentials.refresh(), safe);
            if (safe) {
                return refresh;
            }
            [request, response] = await execute(await credentials.attach({ ...init }, withQs));
            if (response.ok) {
                return unwrap(() => visitor.ok(response), safe);
            }
        }
        return unwrap(() => visitor.fail(request, response), safe);
    }
    return {
        call,
        get: (url, qs, body) => call("GET", url, qs, body),
        post: (url, qs, body) => call("POST", url, qs, body),
        delete: (url, qs, body) => call("DELETE", url, qs, body),
        postBody: (url, body) => call("POST", url, {}, body),
        put: (url, body) => call("PUT", url, {}, body),
    };
}
